import java.net.*;
import java.io.*;
import java.util.Observable;


import javax.swing.*;

/**
 * This is the network connection between server and the client
 * It manages all outgoing and incoming messages to and from the server
 * @author David Couvrette
 * @version 1.0 final release
 */
public class ClientNetwork extends Observable implements Runnable {
  
  private Socket s;
  private OutputStreamWriter out;
  private String msgFromServer;
  private boolean connected = false;
  
  /**
   * Creates new connection with the server
   * @param ip The ip address of the server
   * @param port The port number of the server
   */
  public ClientNetwork(String ip, String port) {
    try {
      s = new Socket(ip, Integer.parseInt(port));
      out = new OutputStreamWriter(s.getOutputStream());
      Thread t = new Thread(this);
      t.start();
      connected = true;
    } catch (Exception e) {
      JOptionPane.showMessageDialog(null, "Connection timed out. Please check your Internet connection and try again.", "Error.ID 1", JOptionPane.WARNING_MESSAGE );
      connected = false;
    }
  }
  
  /**
   * Start the network connection
   */
  public void run() {
    try {
      InputStream in = new BufferedInputStream(s.getInputStream());
      while(true) {
        msgFromServer = "";
        int c = in.read();
        
        while(c != 13) {
          msgFromServer += (char)c;
          c = in.read();
        }
        
        setChanged();
        notifyObservers();
      }
    } catch (IOException e) {
      System.err.println("Error in run");
    }
  }
  
  /**
   * Returns the message received from the server
   */
  public String getMsg() {
    return msgFromServer;
  }
  
  /**
   * Send message to server
   * @param msg Message to send to the server
   */
  public void sendMsg(String msg) {
    try {
      out.write(msg);
      out.flush();
    } catch (IOException e) {
      System.err.println("Couldn't send");
    }
  }
  
  /**
   * Check whether connection has been established with the server
   * @return Is it connected to the server?
   */ 
  public boolean checkConnection() {
    return connected;
  }
}



