import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.BorderFactory; 
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.border.EtchedBorder;

/**
 * Contains the paint board and the "art" tools to draw on the board
 * @author Jung Son
 * @version 1.0 final release
 */ 
public class DrawingPanel extends JPanel{
  
  private ClientNetwork cn;
  public final static int ONE_SECOND = 1000;
  public final static int EXPIRE = 120; // 120 seconds or 2 mintues
  
  private JProgressBar progressBar;
  private javax.swing.Timer timer;
  private JButton startButton, exitButton;
  private ClientFrame cf;
  private PlayScreen ps;
  private JTextField topic = new JTextField(15);
  private Color tempColor = Color.BLACK;
  private DrawingCanvas canvas;
  private int timeElapsed = 0;
  
  /**
   * Construct and initialize the drawing panel
   * @param cf ClientFrame class
   * @param ps PlayScreen class
   * @param cn ClientNetwork class
   */ 
  public DrawingPanel(final ClientFrame cf, final PlayScreen ps, final ClientNetwork cn) {
    Dimension size = new Dimension(600, 500);
    setMaximumSize(size);
    setMinimumSize(size);
    setPreferredSize(size);
    
    setLayout(new BorderLayout());
    
    this.cn = cn;
    this.cf = cf;
    this.ps = ps;
    final String user_name = cf.getUserName();
    
    // show topic in the center
    topic.setHorizontalAlignment(JTextField.CENTER );
    
    // create the paint board and set borders
    canvas = new DrawingCanvas(cn, user_name, ps.getGameMode());
    Border loweredetched = BorderFactory.createEtchedBorder(EtchedBorder.LOWERED);
    TitledBorder title = BorderFactory.createTitledBorder(loweredetched);
    JPanel outerPanel = new JPanel(new BorderLayout());
    outerPanel.setBorder(title);
    outerPanel.add(canvas, BorderLayout.CENTER);
    
    // enable observing of the server messages
    cn.addObserver(canvas);
    add(outerPanel, BorderLayout.CENTER);
    
    // add the necessary components
    JPanel topPanel = new JPanel(new FlowLayout());
    add(topPanel, BorderLayout.NORTH);
    
    JPanel bottomPanel = new JPanel();
    bottomPanel.setLayout(new FlowLayout());

    // add navigation and timer
    JPanel naviPanel = new JPanel(new FlowLayout());
    startButton = new JButton("start");
    if (ps.getGameMode() == ClientModel.ILLUSTRATOR) {
      startButton.addActionListener( new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          if (ps.getNumPlayers() > 1) {
            cn.sendMsg(ClientModel.START + "_\r");
            //request new topic from server.
            ps.enableChat(false);
            if (ps.getGameMode() == ClientModel.ILLUSTRATOR) {
              cn.sendMsg(ClientModel.REQUEST_TOPIC + "_\r");
            } 
          } else {
            JOptionPane.showMessageDialog(null, "You need at least 2 people to play. Please wait for more users to join.", "Error.ID 5", JOptionPane.WARNING_MESSAGE);
          }
        }
      }); 
      bottomPanel.add(startButton);
    }
    
    // add timer
    progressBar = new JProgressBar(0,  EXPIRE);
    progressBar.setValue(0);
    progressBar.setString("TIMER");
    progressBar.setStringPainted(true);
    
    naviPanel.add(progressBar);
    naviPanel.add(Box.createRigidArea(new Dimension(190,0)));
    
    // button to exit the room
    exitButton = new JButton("exit");
    exitButton.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        changeIllustrator();
        cn.sendMsg(ClientModel.LEAVE_GAME + "_" + user_name + "_\r");
        cf.displayWait(cf, cn, user_name);
      }
    });
    bottomPanel.add(exitButton);
    
    // spacers the buttons spread apart
    if (ps.getGameMode() == ClientModel.ILLUSTRATOR) {
      bottomPanel.add(Box.createRigidArea(new Dimension(30,0)));
    } else {
      bottomPanel.add(Box.createRigidArea(new Dimension(210,0)));
    }
    
    // "art" tools
    if (ps.getGameMode() == ClientModel.ILLUSTRATOR) {
      displayIllustratorPanel(canvas, bottomPanel);
    } else {
      displayAssistantPanel(canvas, bottomPanel);
    }
    
    //Create a timer.
    timer = new javax.swing.Timer(ONE_SECOND, new ActionListener() {
      int clock = 0;
      public void actionPerformed(ActionEvent evt) {
        progressBar.setValue(clock);
        if (clock==EXPIRE) {
          clock=0;
          timeElapsed=clock;
            
          if (ps.getGameMode() == ClientModel.ILLUSTRATOR) {
            String tempMsg = "No winner. Correct answer: " + topic.getText();
            cn.sendMsg(ClientModel.CHAT + "_mod_" + tempMsg + "_\r");
            ps.userTalked("mod", tempMsg);
            cn.sendMsg(ClientModel.STOP + "_\r");
          }
          //stopTimer(); 
          
        }
        clock++;
        timeElapsed=clock;
      }
    });
    
    // display area to show the topic
    JLabel topic_label = new JLabel("Topic: ");
    topic.setEditable(false);
    topPanel.add(naviPanel);
    topPanel.add(topic_label);
    topPanel.add(topic);

    add(bottomPanel, BorderLayout.SOUTH);
  }
  
  /*
   * Change to a new illustrator
   */ 
  public void changeIllustrator() {
    if (ps.getGameMode() == ClientModel.ILLUSTRATOR && ps.getNumPlayers() > 1) {
      JTextField[] playerList = ps.getPlayerList();
      Random rand = new Random();
      Boolean found = false;
      
      // randomly select an illustrator until you find one that is not yourself
      while (!found) {
        int spot = rand.nextInt(6);
        
        if (playerList[spot].getText().length() > 0 && (playerList[spot].getText()).compareTo(cf.getUserName()) != 0) {
          cn.sendMsg(ClientModel.NEW_ILLUSTRATOR + "_" + playerList[spot].getText() + "_\r");
          found = true;
          ps.indicateIllustrator(playerList[spot].getText());
          //canvas.clearScreen();
          ps.setGameMode(ClientModel.ASSISTANT);
          cf.setGameMode(ClientModel.ASSISTANT);
          ps.updateDrawingPanel(cf, ps, cn);
        }
      }
    } 
  }
  
  /**
   * Create the assistant's art panel
   * @param canvas Paintboard where the drawing will take place
   * @param buttonPanel Panel to add the buttons to
   */ 
  private void displayAssistantPanel(final DrawingCanvas canvas, JPanel buttonPanel) {
    // buttons to change color
    ButtonGroup group1 = new ButtonGroup();
    makeColorButton("Black", Color.BLACK, group1, buttonPanel, canvas);
    makeColorButton("Red", Color.RED, group1, buttonPanel, canvas);
    makeColorButton("Blue", Color.BLUE, group1, buttonPanel, canvas);
    makeColorButton("Green", Color.GREEN, group1, buttonPanel, canvas);
    makeColorButton("Yellow", Color.YELLOW, group1, buttonPanel, canvas);
  }
  
  /**
   * Create the illustrator's art panel
   * @param canvas Paintboard where the drawing will take place
   * @param buttonPanel Panel to add the buttons to
   */ 
  private void displayIllustratorPanel(final DrawingCanvas canvas, JPanel buttonPanel) {
    ButtonGroup group2 = new ButtonGroup();
    boolean selected = group2.getButtonCount() == 0;
    
    JRadioButton penButton = new JRadioButton("Pen", selected);
    buttonPanel.add(penButton);
    group2.add(penButton);
    penButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          canvas.setEraser(false);
          canvas.setColor(canvas.getColor());
        }
    });
    
    JRadioButton eraseButton = new JRadioButton("Eraser", selected);
    buttonPanel.add(eraseButton);
    group2.add(eraseButton);
    eraseButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          canvas.setEraser(true);
        }
    });
    

    // slider to change paint brush size
    final JSlider sizeSlider = new JSlider(1, 20, 10);
    canvas.setSize(10);
    sizeSlider.addChangeListener(new ChangeListener() {
      public void stateChanged (ChangeEvent event) {
        canvas.setSize(sizeSlider.getValue());
      }
    });
    
    buttonPanel.add(sizeSlider);    
    
    // erase all button
    JButton erase_all = new JButton("Erase All");
    erase_all.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        canvas.clearScreen();
      }
    });
    
    buttonPanel.add(erase_all);
  }
  
  /**
   * Make color changing buttons
   * @param label The name of the button
   * @param c Color of the button
   * @param group Group this button belongs to
   * @param buttonPanel Panel to add this button to
   * @param canvas Paint area where the drawing will take place
   */ 
  private void makeColorButton(String label, final Color c, ButtonGroup group, JPanel buttonPanel, final DrawingCanvas canvas) {
    // select first button in group
    boolean selected = group.getButtonCount() == 0;
    JRadioButton button = new JRadioButton(label, selected);
    buttonPanel.add(button);
    group.add(button);

      button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          canvas.setColor(c);
          cn.sendMsg(ClientModel.COLOR_CHANGE + "_" + c.getRGB() + "_\r");
        }
      });
  }
  
  /**
   * Starts the timer - how long the game will last
   */ 
  public void startTimer() {
    if(ps.getGameMode() == ClientModel.ILLUSTRATOR) {
      canvas.startDrawing();
      startButton.setEnabled(false);
    }
    exitButton.setEnabled(false);
    timer.start();
  }
  
  /**
   * Stops the timer - how long the game will last
   */ 
  public void stopTimer() {
    // reenable chat feature for the illustrator
    if(ps.getGameMode() == ClientModel.ILLUSTRATOR) {
      ps.enableChat(true);
    }
    
    // stop the timer
    timer.stop();
    startButton.setEnabled(true);
    exitButton.setEnabled(true);
    progressBar.setValue(progressBar.getMinimum());
    canvas.disableDrawing();
    canvas.clearScreen();
    changeIllustrator();  
  }
  
  /**
   * Display the topic of the game
   * @param tpc The topic of the game
   */ 
  public void displayTopic(String tpc) {
    topic.setText(tpc);
  }
  
  /**
   * Get the current game topic
   * @return The game topic
   */ 
  public String getTopic() {
    return topic.getText();
  }
  
  /**
   * Get the current time of game play
   * @return The time elapsed
   */ 
  public int getTimeElapsed() {
    return timeElapsed;
  }
  
}