import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import javax.swing.plaf.basic.BasicInternalFrameUI;

/**
 * Displays the log-in screen of the application and establishes connection with 
 * the server using the ClientNetwork
 * @author Jung Son
 * @version 1.0 final release
 */ 
public class LoginScreen extends JInternalFrame {
  
  private ClientNetwork cn;
  private ClientFrame cf;
  private String user_name;
  
  /**
   * Construct and initialize the login screen
   * @param cf The ClientFrame class
   */ 
  public LoginScreen(ClientFrame cf) {
    super("", false, false, false, false);
    
    this.cf = cf;
    
    JPanel content = new JPanel();
    content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
    content.setOpaque(false);
    
    JButton submit = new JButton("Submit");
    
    // create login page contents
    JLabel name_label = new JLabel("Username:");
    content.add(name_label);
    
    final JTextField name_field = new JTextField(15);
    addKeyboardInput(name_field, submit);
    content.add(name_field);
    
    // add spacer
    content.add(Box.createRigidArea(new Dimension(0,5)));
    
    JLabel server_label = new JLabel("Optional Server IP:");
    content.add(server_label);
    
    final JTextField server_field = new JTextField(15);
    addKeyboardInput(server_field, submit);
    content.add(server_field);
    
    // add spacer
    content.add(Box.createRigidArea(new Dimension(0,5)));
    
    JLabel port_label = new JLabel("Optional Server Port:");
    content.add(port_label);
    
    final JTextField port_field = new JTextField(15);
    addKeyboardInput(port_field, submit);
    content.add(port_field);
    
    content.add(Box.createRigidArea(new Dimension(0,5)));
    
    JPanel buttonPane = new JPanel(new FlowLayout());
    
    // create submit button
    buttonPane.add(submit);
    submit.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        connectToServer(name_field.getText(), server_field.getText(), port_field.getText());
      }
    });
    
    // create cancel button
    JButton cancel = new JButton("Cancel");
    buttonPane.add(cancel);
    cancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        System.exit(0);
      }
    });
    
    JPanel pane = new JPanel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    pane.setLayout(gridbag);
    
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(content, c);
    pane.add(content);
    pane.add(buttonPane);
    
    setLayout(new GridBagLayout());
    add(pane);
    
    show();
  }
  
  /**
   * Add the log-in screen to the specified container
   * @param desktop The component to add the log-in screen
   */ 
  public void addJDesktop(JDesktopPane desktop) {
    // disable border and frame top bar
    ((BasicInternalFrameUI)this.getUI()).setNorthPane(null);
    this.setBorder(null);
    
    // add to desktop
    desktop.add(this);
    
    // set internal screen to maximum size
    try {
      this.setMaximum(true);
    } catch(PropertyVetoException e){}; 
  }
  
  /**
   * Establish connection with the server
   * @param user_name The user name specified by the application user
   * @param new_ip Optional ip address to connect to a non-default server
   * @param new_port Optional port number to connect to a non-default server
   */
  public void connectToServer(String user_name, String new_ip, String new_port) {
    String server_ip = ClientModel.DEFAULT_IP;
    String port = ClientModel.DEFAULT_PORT;
    
    System.out.println(new_ip + "   " + new_port);
    
    // if user types in an optional server ip, connect to this server
    if (new_ip != null && new_ip.length() != 0 && new_port != null && new_port.length() != 0) {
      server_ip = new_ip;
      port = new_port;
    } 
    
    // if the user name is no blank, then esbalish connection with the server
    if (user_name != null && user_name.length() != 0) {
      if (user_name.indexOf('_') == -1) {
        cn = new ClientNetwork(server_ip, port);
        if (cn.checkConnection()) {
          cf.setConnection(cn);
          cn.sendMsg(ClientModel.LOGIN + "_" + user_name + "_\r");
        }
      } else {
         JOptionPane.showMessageDialog(null, "Underscore is not a valid character for a user name. Change your user name and try again.", "Error.ID 1", JOptionPane.WARNING_MESSAGE );
      }
    }
  }

  /**
   * Add keyboard input so that user can use the enter key to submit the text forms
   * @param field JTextField to add the keyboard input
   * @param button JButton that is associated with the keyboared input
   */ 
  private void addKeyboardInput(JTextField field, final JButton button) {
    field.addKeyListener(new KeyListener() {
      public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
          button.doClick();
        }
      }
      public void keyTyped(KeyEvent e) {}
      public void keyReleased(KeyEvent e){}
    });
  }
}