import java.net.*;
import java.io.*;
import java.util.*;

/**
 * Holds personal information needs to
 * communicate with a given client.
 * @author David Couvrette
 * @version 1.0 final release
 */
public class Client implements Runnable {
  
  // OutputStream to client
  private OutputStreamWriter out;
  // Socket to client
  private Socket connection;
  // user name
  private String userName;
  // room that this client is in
  private Room whereIAm;
  
  // this, for now, the default message end signal '\r'
  public static final char LINE_RETURN = 13;
  
  /**
   * initializes an output stream to the client
   * @param connection A socket to for the client
   */
  public Client(Socket connection) {
    
    //Temporary client name generator
    Random r = new Random();
    userName = "";
    for (int i = 0; i < 4; i++) {
      userName += (char)(r.nextInt(26) + 97);
    }
    System.out.println(userName);
    this.connection = connection;
    if (connection == null)
      throw new IllegalArgumentException("connection is null");
    try {
      out = new OutputStreamWriter(connection.getOutputStream());
    } catch (IOException e) {
      System.err.println("could not get outputStream fron connection");
    }
  }
  
  /**
   * Sends a message to a client
   * @param message The message that is to be sent
   * @return true if the message was sent succesfully, false if it fails
   */
  public boolean sendMsg(String message) {
    try {
      message += LINE_RETURN;
      out.write(message);
      out.flush();
    } catch (IOException e) {
      return false;
    }
    return true;
  }
  
  /**
   * Gets the user name of the client
   * @return the clients user name
   */
  public String getUserName() {
    return userName;
  }
  
  /**
   * Sets the user name
   * @param userName Sets the user name to the given name
   */
  public void setUserName(String userName) {
    this.userName = userName;
  }
  
  /**
   * Overrided run method that listens for incoming
   * messages from the client.
   * Runs when this thread is started.
   */
  public void run() {
    try {
      BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
      // while (until client disconnects...)
      while(true) {
        String str = "";
        int c = in.read();
        while (c != LINE_RETURN) {
          str += (char)c;
          c = in.read();
        }
        // println for debugging purposes
        //System.out.println("Message recieved from " + getUserName() + ": " + str);
        
        //send message to client's current room
        whereIAm.handleMsg(str, this);
      }
    } catch (IOException e) {
      // client has disconnected from server
      // remove client from server
      System.err.println("Message from client failed: Client Disconnected");
      whereIAm.removeClient(userName);
    }
  }
  
  public void setRoom(Room room) {
    whereIAm = room;
  }
}