import java.io.*;
import java.util.*;

/**
 * Keeps track of the state of the game
 * Generates random topics from a file
 * @version 1.0 final release
 * @author David Couvrette
 */

public class GameState {
  public static final int WAITING_FOR_PLAYERS = 1;
  public static final int IN_GAME = 2;
  public static final String TOPIC_FILE_NAME = "./topics.txt";
  private int gameMode;
  private List topics;
  private String currentTopic;
  
  /**
   * Constructs a GameState
   */
  public GameState() {
    gameMode = WAITING_FOR_PLAYERS;
    topics = new ArrayList();
    Scanner topicFile = null;
    try {
      topicFile = new Scanner(new File(TOPIC_FILE_NAME));
    } catch (FileNotFoundException e) {
      System.err.println("File Not Found");
    }
    while (topicFile.hasNextLine()) {
      topics.add(topicFile.nextLine());
    }
  }
  
  /**
   * Gets a random topic and stores that for when
   * the game starts
   * @return a random topic
   */
  public String getTopic() {
    Random r = new Random();
    int index = r.nextInt(topics.size());
    currentTopic = (String) topics.get(index);
    return currentTopic;
  }
  
  /**
   * Gets the current game mode
   * @return The game mode
   */
  public int getGameMode() {
    return gameMode;
  }
  
  /**
   * Starts the game
   */
  public void startGame() {
    gameMode = IN_GAME;
  }
  
  /**
   * Ends the game
   */
  public void endGame() { 
    gameMode = WAITING_FOR_PLAYERS;
  }
  public static void main(String[] args) {
    GameState s = new GameState();
  }
}