/**
 * Every type of room where a client can be needs
 * to be able to handle incoming messages.
 * @author David Couvrette
 * @version 1.0 final release
 * 
 * Both the game room and the main server room will implement
 * this interface.  Both of these rooms have similarities such
 * as handling messages and adding and removing clients.
 * 
 */
public interface Room {
  
  public static final int ERROR = 0; 
  public static final int DRAW = 70; // sendToAllOthers() done
  public static final int ERASE = 71; // sendToAllOthers() done
  public static final int COLOR_CHANGE = 80; // sendToAllOthers() done
  public static final int SIZE_CHANGE = 81; // sendToAllOthers() done
  public static final int LOGIN = 1;  // done
  public static final int LOGOUT = 2; // done
  public static final int CREATE = 3; // done
  public static final int JOIN = 4; // done
  public static final int START = 5; // done
  public static final int STOP = 6;
  public static final int LEAVE_GAME = 12; // done
  public static final int END_SESSION = 13; // ?
  public static final int GAME_INFO = 14; // done
  public static final int TOPIC_REQUEST = 20; // sendToAllClients() done
  
  public static final int CHAT = 100; // sendToAllOthers() done
  public static final int ADD_USER = 101; // sendToAllOthers() done
  public static final int REMOVE_USER = 102; // sendToAllOthers() done
  public static final int NEW_ILLUSTRATOR = 103; // sendToAllOthers()
  
  /**
   * Handles a message from the client.
   * @param msg message from client
   * @param sender the client that send the message
   */
  public void handleMsg(String msg, Client sender);
  
  /**
   * Adds a new client to the room
   * @param newClient The client that is to be added to the room
   */
  public void addNewClient(Client newClient);
  
  /**
   * Remove a client from the room.
   * This room does not have any direct reference to
   * a client after it has been removed
   * @param userName The name of the client that is being removed
   * @return The client that has just been removed, returns null if
   * the given user name is not in the room
   */
  public Client removeClient(String userName);
  
  /**
   * Checks to see if a given user is in the room.
   * @param userName the user name to search for
   * @return true if the room contains the given user name, otherwise false
   */
  public boolean containsUser(String userName);
}